# Generated by Django 4.2.1 on 2024-02-08 07:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("simple_openid_connect_django", "0002_move_sessions_id_token"),
    ]

    operations = [
        migrations.AddField(
            model_name="openidsession",
            name="raw_id_token",
            field=models.TextField(blank=True),
        ),
    ]
